@@ -24,12 +24,21 @@ import numpy as np
 import pandas as pd
 
 
+from sklearn.base import BaseEstimator, TransformerMixin
+
 from IPython.display import clear_output
-from sklearn.preprocessing import MinMaxScaler, RobustScaler
-from sklearn.preprocessing import KBinsDiscretizer
-from sklearn.preprocessing import KernelCenterer
+from sklearn import preprocessing
+from sklearn.preprocessing import (
+    # MinMaxScaler,
+    RobustScaler,
+    KBinsDiscretizer,
+    KernelCenterer,
+    QuantileTransformer,
+)
 from sklearn.pipeline import Pipeline
 
+from .metrics import eval_information_value
+
 
 class ReplaceValue(BaseEstimator, TransformerMixin):
     """
@@ -287,6 +296,67 @@ class Encoder(BaseEstimator, TransformerMixin):
         return pd.get_dummies(X_in, drop_first=self.drop_first)
 
 
+class OneHotMissingEncoder(BaseEstimator, TransformerMixin):
+    """  """
+
+    def __init__(self, columns, suffix="nan", sep="_", dummy_na=True, drop_last=False):
+        """ """
+        self.columns = columns
+        self.suffix = suffix
+        self.sep = sep
+
+        self.any_missing = None
+        self.column_values = None
+        self.last_value = None
+
+        self.dummy_na = dummy_na
+        self.drop_last = drop_last
+
+    def transform(self, X, **transform_params):
+        """    """
+        X_copy = X.copy()
+
+        final_columns = []
+
+        for col in X_copy.columns:
+            if col not in self.columns:
+                final_columns.append(col)
+            else:
+                for value in self.column_values[col]:
+                    col_name = col + self.sep + str(value)
+                    if (
+                        self.drop_last
+                        and value == self.last_value[col]
+                        and (not self.any_missing[col])
+                    ):
+                        pass  # dropping
+
+                    else:
+                        final_columns.append(col_name)
+                        X_copy[col_name] = (X_copy[col] == value).astype(int)
+
+                if self.any_missing[col]:
+                    if self.dummy_na and not self.drop_last:
+                        col_name = col + self.sep + "nan"
+                        final_columns.append(col_name)
+                        X_copy[col_name] = pd.isnull(X_copy[col]).astype(int)
+
+        return X_copy[final_columns]
+
+    def fit(self, X, y=None, **fit_params):
+        """ """
+        self.any_missing = {col: (pd.notnull(X[col]).sum() > 0) for col in self.columns}
+
+        self.column_values = {
+            col: sorted([x for x in list(X[col].unique()) if pd.notnull(x)])
+            for col in self.columns
+        }
+
+        self.last_value = {col: self.column_values[col][-1] for col in self.columns}
+
+        return self
+
+
 class MeanModeImputer(BaseEstimator, TransformerMixin):
     """
 
@@ -372,7 +442,7 @@ class ScalerDF(BaseEstimator, TransformerMixin):
 
     def __transformation(self, X_in):
         X = X_in.copy()
-        scaler = MinMaxScaler(copy=True, feature_range=(0, 1))
+        scaler = preprocessing.MinMaxScaler(copy=True, feature_range=(0, 1))
 
         try:
             ind = np.array(list(X.index)).reshape(-1, 1)
@@ -394,6 +464,20 @@ class ScalerDF(BaseEstimator, TransformerMixin):
         return X
 
 
+class MinMaxScaler(preprocessing.MinMaxScaler):
+    def __init__(self, **kwargs):
+
+        super().__init__(**kwargs)
+
+    def fit(self, X, y=None):
+
+        super().fit(X)
+
+    def transform(self, X):
+
+        return pd.DataFrame(super().transform(X), columns=X.columns, index=X.index)
+
+
 class DataFrameImputer(TransformerMixin):
     def __init__(self):
         """
@@ -510,48 +594,28 @@ class EncoderDataframe(TransformerMixin):
 from sklearn.preprocessing import OrdinalEncoder
 
 
-class OrdinalEncoderDataframe(TransformerMixin):
-    """
-    >>> enc = OrdinalEncoderDataframe(feature_categories = {
-                'ExterQual':['Po','Fa','TA','Gd','Ex'],
-                'ExterCond':['Po','Fa','TA','Gd','Ex']})
-    >>> X = enc.fit_transform(X)
-    """
-
-    def __init__(self, feature_categories):
-        self.feature_categories = feature_categories
-
-    def fit(self, X, y=None):
-        self.enc = {}
-        for feature_name in self.feature_categories.keys():
-            self.enc[feature_name] = OrdinalEncoder(
-                [self.feature_categories[feature_name]]
-            )
-            self.enc[feature_name].fit(
-                np.array(X_trn[feature_name].values).reshape(-1, 1)
-            )
-        return self
-
-    def transform(self, X, y=None):
-        X_ = X.copy()
-
-        for feature_name in self.feature_categories.keys():
-            X_[feature_name] = self.enc[feature_name].transform(
-                np.array(X_[feature_name].values).reshape(-1, 1)
-            )
-
-        return X_
-
-
 class NumericBinner(BaseEstimator, TransformerMixin):
     """  """
 
-    def __init__(self, columns=None, max_bins=8):
+    def __init__(self, columns=None, min_bins=None, max_bins=None, n_bins=None):
+
         """ """
         self.columns = columns
         self.binner = None
         self.bins = None
+        self.min_bins = min_bins
         self.max_bins = max_bins
+        self.n_bins = n_bins
+
+        if (self.min_bins is None) and (self.max_bins is None) and (n_bins is None):
+            self.min_bins = 2
+            self.max_bins = 8
+
+        elif (
+            (self.min_bins is None) and (self.max_bins is None) and (n_bins is not None)
+        ):
+            self.min_bins = self.n_bins
+            self.max_bins = self.n_bins
 
     def transform(self, X, **transform_params):
         """    """
@@ -642,11 +706,11 @@ class NumericBinner(BaseEstimator, TransformerMixin):
                     binner_temp = binner
 
                 clear_output()
-
-                # self.bins[column_name] = eval_information_value(X_local[column_name],y,y_values = [0,1],goods = 0)
-
             self.bins[column_name] = bin_temp
             self.binner[column_name] = binner_temp
-            # X_temp[column_name] = X_temp[column_name].apply(lambda x: self.bins[column_name]['woe'][x])
 
         return self
+
+    def fit_transform(self, X, y):
+        self.fit(X, y)
+        return self.transform(X)
